unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    ScanTime: TTimer;
    MasterPanel: TPanel;
    ErrLabel: TLabel;
    StartStopButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ScanTimeTimer(Sender: TObject);
    procedure StartStopButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB, TintProgresses;

var
  Ch: array of record
    ChannelPanel: TPanel;
    ChannelLabel: TLabel;
    ChannelProgress: TTintProgress;
  end;
  DI: LongWord;

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
  Err, L, PID: LongWord;
  V, Hz: Double;
  Config: array of Byte;
  RangeCode: Byte;

begin
  SetLength(Ch, 16);
  for I := 0 to High(Ch) do with Ch[I] do begin
    ChannelPanel := TPanel.Create(Self);
    with ChannelPanel do begin
      BevelOuter := bvNone;
      Top := ErrLabel.Height + 16 * I;
      Height := 16;
      Align := alTop;
      Parent := Self;
    end;
    ChannelLabel := TLabel.Create(Self);
    with ChannelLabel do begin
      AutoSize := False;
      Width := 80;
      Align := alLeft;
      Parent := ChannelPanel;
    end;
    ChannelProgress := TTintProgress.Create(Self);
    with ChannelProgress do begin
      Align := alClient;
      Parent := ChannelPanel;
    end;
  end;


  DI := ResolveDeviceIndex(diOnly);
  if DI = diNone then begin
    ErrLabel.Caption := 'Error: Failed to resolve diOnly.';
    Exit;
  end;

  Err := QueryDeviceInfo(DI, @PID, nil, nil, nil, nil);
  if Err <> ERROR_SUCCESS then begin
    ErrLabel.Caption := 'Error: QueryDeviceInfo returned error ' + IntToStr(Err) + '.';
    Exit;
  end;

  case PID of
    $8040..$8044, $8140..$8144: ;
    else begin
      ErrLabel.Caption := 'Error: Device found(PID ' + IntToHex(PID, 4) + ') is not a USB-AIxxx-xxx.';
      Exit;
    end;
  end;

  ADC_SetCal(DI, ':AUTO:');

  L := 21;
  SetLength(Config, L);
  for I := 0 to High(Config) do Config[I] := 0;

  for I := 0 to 15 do Config[I] := $01; //Set all channels to bipolar 10V.
  //Config[$11] is set by the init.
  Config[$12] := $F0; //Select all 16 channels.
  Config[$13] := 7; //Oversample +7. Max oversample for BG timer acq is +30.
  ADC_SetConfig(DI, @Config[0], L);

  Err := ADC_CSA_InitFastScanV(DI);
  if Err <> ERROR_SUCCESS then begin
    ErrLabel.Caption := 'Error: ADC_CSA_InitFastScanV returned error ' + IntToStr(Err) + '.';
    Exit;
  end;
  ADC_GetConfig(DI, @Config[0], L);

  Hz := 100;
  CTR_StartOutputFreq(DI, 0, @Hz);

  for I := 0 to High(Ch) do with Ch[I] do begin
    RangeCode := Config[I div 8];
    V := 1;
    if (RangeCode and 2) = 0 then V := V * 2;
    if (RangeCode and 4) = 0 then V := V * 5;
    if (RangeCode and 1) <> 0 then
      ChannelProgress.Min := Round(-1000 * V)
    else
      ChannelProgress.Min := 0
    ;
    ChannelProgress.Max := Round(+1000 * V);
    ChannelProgress.Position := 0;
  end;

  ErrLabel.Caption := 'OK: Found PID ' + IntToHex(PID, 4) + ' at index ' + IntToStr(DI) + '.';
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  Err: LongWord;
  Hz: Double;

begin
  ScanTime.Enabled := False;

  Hz := 1;
  CTR_StartOutputFreq(DI, 0, @Hz);

  //Optional, puts the config back the way it was before ADC_InitFastITScanV.
  Err := ADC_CSA_ResetFastScanV(DI);
end;

procedure TForm1.StartStopButtonClick(Sender: TObject);
begin
  if ScanTime.Enabled then begin
    StartStopButton.Caption := '&Start';
    ScanTime.Enabled := False;
  end
  else begin
    StartStopButton.Caption := '&Stop';
    ScanTime.Enabled := True;
  end
end;

procedure TForm1.ScanTimeTimer(Sender: TObject);
var
  I: Integer;
  Err: LongWord;
  ADBuf: array of Double;
  V: Double;

begin
  SetLength(ADBuf, 128);
  for I := 0 to High(ADBuf) do ADBuf[I] := MAXINT;

  Err := ADC_CSA_GetFastScanV(DI, @ADBuf[0]);
  if Err <> ERROR_SUCCESS then begin
    ErrLabel.Caption := 'Error: ADC_CSA_GetFastScanV returned error ' + IntToStr(Err) + '.';
    Exit;
  end;

  for I := 0 to High(Ch) do with Ch[I] do begin
    V := ADBuf[I];
    if V = MAXINT then begin
      ChannelLabel.Caption := 'Ch ' + IntToStr(I) + ': ???';
      ChannelProgress.Position := 0;
    end
    else begin
      ChannelLabel.Caption := 'Ch ' + IntToStr(I) + ': ' + Format('%g', [ Round(V * 1000)/1000 ]);
      ChannelProgress.Position := Round(1000 * V);
    end;
  end;
end;

end.
